"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseRequestInput = parseRequestInput;
exports.responseValidationFactory = responseValidationFactory;
const errors_1 = require("./errors");
function parseRequestInput(schema, input, type) {
    try {
        return schema?.parse(input);
    }
    catch (err) {
        throw errors_1.KoaRuntimeError.RequestError(err, type);
    }
}
// TODO: optional response validation
function responseValidationFactory(possibleResponses, defaultResponse) {
    // Exploit the natural ordering matching the desired specificity of eg: 404 vs 4xx
    possibleResponses.sort((x, y) => (x[0] < y[0] ? -1 : 1));
    return (status, value) => {
        try {
            for (const [match, schema] of possibleResponses) {
                const isMatch = (/^\d+$/.test(match) && String(status) === match) ||
                    (/^\d[xX]{2}$/.test(match) && String(status)[0] === match[0]);
                if (isMatch) {
                    return schema.parse(value);
                }
            }
            if (defaultResponse) {
                return defaultResponse.parse(value);
            }
            // TODO: throw on unmatched response
            return value;
        }
        catch (err) {
            throw errors_1.KoaRuntimeError.ResponseError(err);
        }
    };
}
//# sourceMappingURL=zod.js.map